@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Fanout, Inc.
+ * Copyright (C) 2012-2017 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
@@ -32,6 +32,7 @@
 #include "zhttpresponsepacket.h"
 #include "log.h"
 #include "zutil.h"
+#include "logutil.h"
 
 #define OUT_HWM 100
 #define IN_HWM 100
@@ -345,14 +346,14 @@ public:
 		if(client_out_sock)
 		{
 			if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-				log_debug("%s client: OUT %s", logprefix, qPrintable(TnetString::variantToString(vpacket, -1).mid(0, 1000)));
+				LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, vpacket, "body", "%s client: OUT", logprefix);
 
 			client_out_sock->write(QList<QByteArray>() << buf);
 		}
 		else
 		{
 			if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-				log_debug("%s client req: OUT %s", logprefix, qPrintable(TnetString::variantToString(vpacket, -1).mid(0, 1000)));
+				LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, vpacket, "body", "%s client req: OUT", logprefix);
 
 			client_req_sock->write(QList<QByteArray>() << QByteArray() << buf);
 		}
@@ -367,7 +368,7 @@ public:
 		QByteArray buf = QByteArray("T") + TnetString::fromVariant(vpacket);
 
 		if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-			log_debug("%s client: OUT %s %s", logprefix, instanceAddress.data(), qPrintable(TnetString::variantToString(vpacket, -1).mid(0, 1000)));
+			LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, vpacket, "body", "%s client: OUT %s", logprefix, instanceAddress.data());
 
 		QList<QByteArray> msg;
 		msg += instanceAddress;
@@ -385,7 +386,7 @@ public:
 		QByteArray buf = instanceAddress + " T" + TnetString::fromVariant(vpacket);
 
 		if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-			log_debug("%s server: OUT %s %s", logprefix, instanceAddress.data(), qPrintable(TnetString::variantToString(vpacket, -1).mid(0, 1000)));
+			LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, vpacket, "body", "%s server: OUT %s", logprefix, instanceAddress.data());
 
 		server_out_sock->write(QList<QByteArray>() << buf);
 	}
@@ -510,7 +511,7 @@ public slots:
 		}
 
 		if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-			log_debug("zhttp/zws client: IN %s %s", receiver.data(), qPrintable(TnetString::variantToString(data, -1).mid(0, 1000)));
+			LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, data, "body", "zhttp/zws client: IN %s", receiver.data());
 
 		ZhttpResponsePacket p;
 		if(!p.fromVariant(data))
@@ -581,7 +582,7 @@ public slots:
 		}
 
 		if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-			log_debug("zhttp/zws server: IN %s", qPrintable(TnetString::variantToString(data, -1).mid(0, 1000)));
+			LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, data, "body", "zhttp/zws server: IN");
 
 		ZhttpRequestPacket p;
 		if(!p.fromVariant(data))
@@ -694,7 +695,7 @@ public slots:
 			}
 
 			if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-				log_debug("zhttp/zws client req: IN %s", qPrintable(TnetString::variantToString(data, -1).mid(0, 1000)));
+				LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, data, "body", "zhttp/zws client req: IN");
 
 			ZhttpResponsePacket p;
 			if(!p.fromVariant(data))
@@ -749,7 +750,7 @@ public slots:
 		}
 
 		if(log_outputLevel() >= LOG_LEVEL_DEBUG)
-			log_debug("zhttp/zws server: IN stream %s", qPrintable(TnetString::variantToString(data, -1).mid(0, 1000)));
+			LogUtil::logVariantWithContent(LOG_LEVEL_DEBUG, data, "body", "zhttp/zws server: IN stream");
 
 		ZhttpRequestPacket p;
 		if(!p.fromVariant(data))
