@@ -227,6 +227,12 @@ public:
 		trimlist(&m2a_in_stream_specs);
 		QStringList m2a_out_specs = settings.value("handler/m2a_out_specs").toStringList();
 		trimlist(&m2a_out_specs);
+		QStringList intreq_out_specs = settings.value("handler/proxy_intreq_out_specs").toStringList();
+		trimlist(&intreq_out_specs);
+		QStringList intreq_out_stream_specs = settings.value("handler/proxy_intreq_out_stream_specs").toStringList();
+		trimlist(&intreq_out_stream_specs);
+		QStringList intreq_in_specs = settings.value("handler/proxy_intreq_in_specs").toStringList();
+		trimlist(&intreq_in_specs);
 		QString proxy_inspect_spec = settings.value("handler/proxy_inspect_spec").toString();
 		QString proxy_accept_spec = settings.value("handler/proxy_accept_spec").toString();
 		QString proxy_retry_out_spec = settings.value("handler/proxy_retry_out_spec").toString();
@@ -244,6 +250,9 @@ public:
 		bool ok;
 		int ipcFileMode = settings.value("handler/ipc_file_mode", -1).toString().toInt(&ok, 8);
 		bool shareAll = settings.value("handler/share_all").toBool();
+		int messageRate = settings.value("handler/message_rate", -1).toInt();
+		int messageHwm = settings.value("handler/message_hwm", -1).toInt();
+		int messageBlockSize = settings.value("handler/message_block_size", -1).toInt();
 
 		if(m2a_in_stream_specs.isEmpty() || m2a_out_specs.isEmpty())
 		{
@@ -264,6 +273,9 @@ public:
 		config.instanceId = "pushpin-handler_" + QByteArray::number(QCoreApplication::applicationPid());
 		config.serverInStreamSpecs = m2a_in_stream_specs;
 		config.serverOutSpecs = m2a_out_specs;
+		config.clientOutSpecs = intreq_out_specs;
+		config.clientOutStreamSpecs = intreq_out_stream_specs;
+		config.clientInSpecs = intreq_in_specs;
 		config.inspectSpec = proxy_inspect_spec;
 		config.acceptSpec = proxy_accept_spec;
 		config.retryOutSpec = proxy_retry_out_spec;
@@ -280,6 +292,9 @@ public:
 		config.pushInHttpPort = push_in_http_port;
 		config.ipcFileMode = ipcFileMode;
 		config.shareAll = shareAll;
+		config.messageRate = messageRate;
+		config.messageHwm = messageHwm;
+		config.messageBlockSize = messageBlockSize;
 
 		engine = new Engine(this);
 		if(!engine->start(config))
