@@ -25,6 +25,7 @@
 #include <QByteArray>
 #include <QList>
 #include <QHash>
+#include <QUrl>
 #include "packet/httpresponsedata.h"
 
 class Instruct
@@ -53,11 +54,14 @@ public:
 	int keepAliveTimeout;
 	QHash<QString, QString> meta;
 	HttpResponseData response;
+	QUrl nextLink;
+	int nextLinkTimeout;
 
 	Instruct() :
 		holdMode(NoHold),
 		timeout(-1),
-		keepAliveTimeout(-1)
+		keepAliveTimeout(-1),
+		nextLinkTimeout(-1)
 	{
 	}
 
