@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Fanout, Inc.
+ * Copyright (C) 2014-2017 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
@@ -49,25 +49,29 @@ static bool validate_token(const QByteArray &token, const QByteArray &key)
 
 namespace ProxyUtil {
 
-bool manipulateRequestHeaders(const char *logprefix, void *object, HttpRequestData *requestData, const QByteArray &defaultUpstreamKey, const DomainMap::Entry &entry, const QByteArray &sigIss, const QByteArray &sigKey, bool acceptXForwardedProtocol, bool useXForwardedProtocol, const XffRule &xffTrustedRule, const XffRule &xffRule, const QList<QByteArray> &origHeadersNeedMark, const QHostAddress &peerAddress, const InspectData &idata, bool stripHeaders)
+// check if the request is coming from a grip proxy already
+bool checkTrustedClient(const char *logprefix, void *object, const HttpRequestData &requestData, const QByteArray &defaultUpstreamKey)
 {
-	// check if the request is coming from a grip proxy already
-	bool trustedClient = false;
 	if(!defaultUpstreamKey.isEmpty())
 	{
-		QByteArray token = requestData->headers.get("Grip-Sig");
+		QByteArray token = requestData.headers.get("Grip-Sig");
 		if(!token.isEmpty())
 		{
 			if(validate_token(token, defaultUpstreamKey))
-			{
-				log_debug("%s: %p passing to upstream", logprefix, object);
-				trustedClient = true;
-			}
-			else
-				log_debug("%s: %p signature present but invalid: %s", logprefix, object, token.data());
+				return true;
+
+			log_debug("%s: %p signature present but invalid: %s", logprefix, object, token.data());
 		}
 	}
 
+	return false;
+}
+
+void manipulateRequestHeaders(const char *logprefix, void *object, HttpRequestData *requestData, bool trustedClient, const DomainMap::Entry &entry, const QByteArray &sigIss, const QByteArray &sigKey, bool acceptXForwardedProtocol, bool useXForwardedProtocol, const XffRule &xffTrustedRule, const XffRule &xffRule, const QList<QByteArray> &origHeadersNeedMark, const QHostAddress &peerAddress, const InspectData &idata, bool stripHeaders)
+{
+	if(trustedClient)
+		log_debug("%s: %p passing to upstream", logprefix, object);
+
 	if(!trustedClient && entry.origHeaders)
 	{
 		// copy headers to include magic prefix, so that the original
@@ -163,11 +167,16 @@ bool manipulateRequestHeaders(const char *logprefix, void *object, HttpRequestDa
 	}
 
 	requestData->headers.removeAll("Grip-Feature");
-	requestData->headers += HttpHeader("Grip-Feature", "status");
+	requestData->headers += HttpHeader("Grip-Feature",
+		"status, session, link:next, filter:skip-self, filter:skip-users, filter:require-sub, filter:build-id");
 
 	if(!idata.sid.isEmpty())
 	{
 		requestData->headers += HttpHeader("Grip-Session-Id", idata.sid);
+	}
+
+	if(!idata.lastIds.isEmpty())
+	{
 		QHashIterator<QByteArray, QByteArray> it(idata.lastIds);
 		while(it.hasNext())
 		{
@@ -211,8 +220,6 @@ bool manipulateRequestHeaders(const char *logprefix, void *object, HttpRequestDa
 		xffValues += peerAddress.toString().toUtf8();
 	if(!xffValues.isEmpty())
 		requestData->headers += HttpHeader("X-Forwarded-For", HttpHeaders::join(xffValues));
-
-	return trustedClient;
 }
 
 void applyHost(QUrl *url, const QString &host)
@@ -243,4 +250,27 @@ void applyHostHeader(HttpHeaders *headers, const QUrl &uri)
 	}
 }
 
+QString targetToString(const DomainMap::Target &target)
+{
+	if(target.type == DomainMap::Target::Test)
+		return "test";
+	else if(target.type == DomainMap::Target::Custom)
+		return(target.zhttpRoute.req ? "zhttpreq/" : "zhttp/") + target.zhttpRoute.baseSpec;
+	else // Default
+		return target.connectHost + ':' + QString::number(target.connectPort);
+}
+
+QHostAddress getLogicalAddress(const HttpHeaders &headers, const XffRule &xffRule, const QHostAddress &peerAddress)
+{
+	QList<QByteArray> xffValues = headers.getAll("X-Forwarded-For");
+	if(!xffValues.isEmpty() && xffRule.truncate != 0)
+	{
+		QHostAddress addr;
+		if(addr.setAddress(QString::fromUtf8(xffValues.first())))
+			return addr;
+	}
+
+	return peerAddress;
+}
+
 }
